#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.zmi.paragraph module

This module defines components which are required to handle external
videos paragraphs management interface.
"""

from pyramid.decorator import reify
from pyramid.events import subscriber
from pyramid.httpexceptions import HTTPBadRequest
from pyramid.response import Response
from pyramid.view import view_config
from zope.interface import Interface, Invalid, alsoProvides

from pyams_content.component.paragraph import IParagraphContainer, IParagraphContainerTarget
from pyams_content.component.paragraph.zmi import BaseParagraphAddForm, BaseParagraphAddMenu, \
    IInnerParagraphEditForm, IParagraphContainerBaseTable, InnerParagraphPropertiesEditForm, \
    ParagraphPropertiesEditFormMixin
from pyams_content.component.video import IExternalVideoProvider, IExternalVideoSettings, external_video_settings
from pyams_content.component.video.interfaces import EXTERNAL_VIDEO_PARAGRAPH_ICON_CLASS, EXTERNAL_VIDEO_PARAGRAPH_NAME, \
    EXTERNAL_VIDEO_PARAGRAPH_TYPE, IExternalVideoParagraph
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces.form import IDataExtractedEvent, IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_layer.skin import apply_skin
from pyams_portal.zmi.widget import RendererSelectFieldWidget
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_utils.adapter import adapter_config
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.interfaces.form import NO_VALUE_STRING
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import RawContentProvider, viewlet_config
from pyams_zmi.form import AdminAddForm, AdminEditForm, AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager
from pyams_zmi.skin import AdminSkin

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='add-external-video-paragraph.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerBaseTable,
                manager=IContextAddingsViewletManager, weight=75)
class ExternalVideoParagraphAddMenu(BaseParagraphAddMenu):
    """External video paragraph add menu"""

    label = EXTERNAL_VIDEO_PARAGRAPH_NAME
    icon_class = EXTERNAL_VIDEO_PARAGRAPH_ICON_CLASS

    factory_name = EXTERNAL_VIDEO_PARAGRAPH_TYPE
    href = 'add-external-video-paragraph.html'


@ajax_form_config(name='add-external-video-paragraph.html',
                  context=IParagraphContainer, layer=IPyAMSLayer)
class ExternalVideoParagraphAddForm(BaseParagraphAddForm):
    """External video paragraph add form"""

    @property
    def fields(self):
        fields = Fields(IExternalVideoParagraph).select('title', 'author', 'description',
                                                        'provider_name', 'renderer')
        provider_name = self.request.params.get('addform.widgets.provider_name')
        if provider_name:
            provider = get_utility(IExternalVideoProvider, name=provider_name)
            fields += Fields(provider.settings_interface)
        return fields

    content_factory = IExternalVideoParagraph

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        provider_name = self.widgets.get('provider_name')
        if provider_name is not None:
            provider_name.required = True
            provider_name.prompt = True
            provider_name.prompt_message = _("Select video provider...")
            provider_name.object_data = {
                'ams-change-handler': 'MyAMS.helpers.select2ChangeHelper',
                'ams-stop-propagation': 'true',
                'ams-select2-helper-type': 'html',
                'ams-select2-helper-url': absolute_url(self.context, self.request,
                                                       'get-video-provider-settings-add-form.html'),
                'ams-select2-helper-argument': 'provider_name',
                'ams-select2-helper-target': '#video-settings-helper'
            }
            provider_name.suffix = RawContentProvider(html='<div id="video-settings-helper" '
                                                      '     class="py-2 no-footer"></div>')
            alsoProvides(provider_name, IObjectData)


@subscriber(IDataExtractedEvent, form_selector=ExternalVideoParagraphAddForm)
def handle_video_paragraph_add_form_data_extraction(event):
    """Handle provider name data extraction"""
    data = event.data
    if not data.get('provider_name'):
        event.form.widgets.errors += (Invalid(_("Video provider is required")),)


class ExternalVideoProviderSettingsAddForm(AdminAddForm):
    """External video provider settings add form"""

    buttons = Buttons(Interface)

    def __init__(self, context, request, provider):
        super().__init__(context, request)
        self.provider = provider


@adapter_config(required=(IParagraphContainer, IPyAMSLayer, ExternalVideoProviderSettingsAddForm),
                provides=IGroup)
class ExternalVideoProviderSettingsAddFormGroup(Group):
    """External video provider settings add form group"""

    legend = _("Provider settings")
    prefix = BaseParagraphAddForm.prefix

    @reify
    def fields(self):
        return Fields(self.parent_form.provider.settings_interface)


@view_config(name='get-video-provider-settings-add-form.html',
             context=IParagraphContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, xhr=True)
def video_provider_settings_add_form(request):
    """Video provider settings add form"""
    apply_skin(request, AdminSkin)
    provider_name = request.params.get('provider_name')
    if provider_name is None:
        raise HTTPBadRequest("Missing provider name argument")
    if (not provider_name) or (provider_name == NO_VALUE_STRING):
        return Response('')
    provider = get_utility(IExternalVideoProvider, name=provider_name)
    form = ExternalVideoProviderSettingsAddForm(request.context, request, provider)
    form.update()
    return Response(form.render())


#
# External video paragraph edit form
#

class ExternalVideoParagraphPropertiesEditFormMixin(ParagraphPropertiesEditFormMixin):
    """External video paragraph propertied edit form mixin"""

    @reify
    def fields(self):
        fields = Fields(IExternalVideoParagraph).select('title', 'author', 'description', 'provider_name',
                                                        'renderer')
        fields['renderer'].widget_factory = RendererSelectFieldWidget
        provider_name = self.request.params.get(f'{self.prefix}widgets.provider_name')
        if provider_name:
            provider = get_utility(IExternalVideoProvider, name=provider_name)
            fields += Fields(provider.settings_interface)
        return fields

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        provider_name = self.widgets.get('provider_name')
        if provider_name is not None:
            key = ICacheKeyValue(self.context)
            provider_name.required = True
            provider_name.prompt = True
            provider_name.prompt_message = _("Select video provider...")
            provider_name.object_data = {
                'ams-change-handler': 'MyAMS.helpers.select2ChangeHelper',
                'ams-stop-propagation': 'true',
                'ams-select2-helper-type': 'html',
                'ams-select2-helper-url': absolute_url(self.context, self.request,
                                                       'get-video-provider-settings-edit-form.html'),
                'ams-select2-helper-argument': 'provider_name',
                'ams-select2-helper-target': f'#video-settings-helper-{key}'
            }
            alsoProvides(provider_name, IObjectData)
            provider = self.context.get_provider()
            group = ExternalVideoProviderSettingsEditFormGroup(self.context, self.request, self, provider)
            group.update()
            provider_name.suffix = RawContentProvider(html=f'<div id="video-settings-helper-{key}" '
                                                           f'     class="py-2 no-footer">'
                                                           f'    {group.render()}'
                                                           f'</div>')


class ExternalVideoProviderSettingsEditForm(AdminEditForm):
    """External video provider settings edit form"""

    buttons = Buttons(Interface)
    hide_section = True

    def __init__(self, context, request, provider):
        super().__init__(context, request)
        self.provider = provider

    @property
    def prefix(self):
        return f'form_{self.context.__name__}.'


@adapter_config(required=(IExternalVideoSettings, IPyAMSLayer, ExternalVideoProviderSettingsEditForm),
                provides=IGroup)
class ExternalVideoProviderSettingsEditFormGroup(Group):
    """External video provider settings edit form group"""

    legend = _("Provider settings")

    def __init__(self, context, request, parent_form, provider=None):
        super().__init__(context, request, parent_form)
        self.provider = provider if provider is not None else parent_form.provider

    @property
    def prefix(self):
        video = get_parent(self.context, IExternalVideoParagraph)
        return f'form_{video.__name__}.'

    @reify
    def fields(self):
        return Fields(self.provider.settings_interface)


@adapter_config(required=(IExternalVideoParagraph, IAdminLayer),
                provides=IInnerParagraphEditForm)
class ExternalVideoParagraphInnerPropertiesEditForm(ExternalVideoParagraphPropertiesEditFormMixin,
                                                    InnerParagraphPropertiesEditForm):
    """External video paragraph inner properties edit form"""


@ajax_form_config(name='properties.html',
                  context=IExternalVideoParagraph, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ExternalVideoParagraphPropertiesEditForm(ExternalVideoParagraphPropertiesEditFormMixin,
                                               AdminModalEditForm):
    """External video paragraph properties edit form"""

    @property
    def provider(self):
        return self.context.get_provider()


@view_config(name='get-video-provider-settings-edit-form.html',
             context=IExternalVideoParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, xhr=True)
def video_provider_settings_edit_form(request):
    """Video provider settings edit form"""
    apply_skin(request, AdminSkin)
    provider_name = request.params.get('provider_name')
    if provider_name is None:
        raise HTTPBadRequest("Missing provider name argument")
    if (not provider_name) or (provider_name == NO_VALUE_STRING):
        return Response('')
    settings = external_video_settings(request.context, provider_name)
    provider = get_utility(IExternalVideoProvider, name=provider_name)
    form = ExternalVideoProviderSettingsEditForm(settings, request, provider)
    form.update()
    return Response(form.render())



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.portlet.skin module

This module defines several renderers of view items portlet.
"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.links import IInternalLink
from pyams_content.shared.view.portlet import IViewItemsPortletSettings
from pyams_content.shared.view.portlet.skin.interfaces import HEADER_DISPLAY_MODE, IViewItemTargetURL, \
    IViewItemsPortletHorizontalRendererSettings, IViewItemsPortletPanelsRendererSettings, \
    IViewItemsPortletVerticalRendererSettings
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.skin import PortletRenderer
from pyams_sequence.reference import InternalReferenceMixin
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.text import get_text_start
from pyams_utils.url import canonical_url, relative_url

__docformat__ = 'restructuredtext'

from pyams_content import _


class BaseViewItemsPortletRenderer(PortletRenderer):
    """Base view items portlet renderer"""

    @staticmethod
    def is_internal_link(link):
        """Internal link checker"""
        return IInternalLink.providedBy(link)

    def get_url(self, target):
        """Item URL getter"""
        target_url = self.request.registry.queryMultiAdapter((target, self.request),
                                                             IViewItemTargetURL)
        if target_url is not None:
            if target_url.target is None:
                return target_url.url
            target = target_url.target
        if self.settings.force_canonical_url:
            return canonical_url(target, self.request)
        return relative_url(target, self.request)

    def render(self, template_name=''):
        result = super().render(template_name)
        if self.settings.first_page_only:
            start = int(self.request.params.get('start', 0))
            if start:
                return ''
        return result


#
# Vertical list view items renderer
#

@factory_config(IViewItemsPortletVerticalRendererSettings)
class ViewItemsPortletVerticalRendererSettings(InternalReferenceMixin, Persistent, Contained):
    """View items portlet vertical renderer settings"""

    display_illustrations = FieldProperty(IViewItemsPortletVerticalRendererSettings['display_illustrations'])
    thumb_selection = FieldProperty(IViewItemsPortletVerticalRendererSettings['thumb_selection'])
    display_breadcrumbs = FieldProperty(IViewItemsPortletVerticalRendererSettings['display_breadcrumbs'])
    display_tags = FieldProperty(IViewItemsPortletVerticalRendererSettings['display_tags'])
    paginate = FieldProperty(IViewItemsPortletVerticalRendererSettings['paginate'])
    page_size = FieldProperty(IViewItemsPortletVerticalRendererSettings['page_size'])
    reference = FieldProperty(IViewItemsPortletVerticalRendererSettings['reference'])
    link_label = FieldProperty(IViewItemsPortletVerticalRendererSettings['link_label'])


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/view-list-vertical.pt', layer=IPyAMSLayer)
class ViewItemsPortletVerticalRenderer(BaseViewItemsPortletRenderer):
    """View items portlet vertical renderer"""

    label = _("Simple vertical list (default)")
    weight = 1

    settings_interface = IViewItemsPortletVerticalRendererSettings


#
# Horizontal list view items renderer
#

@factory_config(IViewItemsPortletHorizontalRendererSettings)
class ViewItemsPortletHorizontalRendererSettings(Persistent, Contained):
    """View items portlet horizontal renderer settings"""

    thumb_selection = FieldProperty(IViewItemsPortletHorizontalRendererSettings['thumb_selection'])


@adapter_config(name='horizontal',
                required=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/view-list-horizontal.pt', layer=IPyAMSLayer)
class ViewItemsPortletHorizontalRenderer(BaseViewItemsPortletRenderer):
    """View items portlet horizontal renderer"""

    label = _("Horizontal thumbnails list")
    weight = 10

    settings_interface = IViewItemsPortletHorizontalRendererSettings


#
# Panels view items renderer
#

@factory_config(IViewItemsPortletPanelsRendererSettings)
class ViewItemsPortletPanelsRendererSettings(Persistent, Contained):
    """View items portlet panels renderer settings"""

    display_illustrations = FieldProperty(IViewItemsPortletPanelsRendererSettings['display_illustrations'])
    thumb_selection = FieldProperty(IViewItemsPortletPanelsRendererSettings['thumb_selection'])
    paginate = FieldProperty(IViewItemsPortletPanelsRendererSettings['paginate'])
    page_size = FieldProperty(IViewItemsPortletPanelsRendererSettings['page_size'])
    header_display_mode = FieldProperty(IViewItemsPortletPanelsRendererSettings['header_display_mode'])
    start_length = FieldProperty(IViewItemsPortletPanelsRendererSettings['start_length'])


@adapter_config(name='panels',
                required=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/view-panels.pt', layer=IPyAMSLayer)
class ViewItemsPortletPanelsRenderer(BaseViewItemsPortletRenderer):
    """View items portlet panels renderer"""

    label = _("Three vertical panels with panoramic illustrations")
    weight = 30

    settings_interface = IViewItemsPortletPanelsRendererSettings

    def get_header(self, item):
        settings = self.renderer_settings
        display_mode = settings.header_display_mode
        if display_mode == HEADER_DISPLAY_MODE.HIDDEN.value:
            return ''
        header = II18n(item).query_attribute('header', request=self.request)
        if display_mode == HEADER_DISPLAY_MODE.START.value:
            header = get_text_start(header, settings.start_length)
        return header



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.navigation.portlet.interfaces module

This module defines navigation menus portlet interfaces.
"""

from zope.container.constraints import containers, contains
from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice

from pyams_content.component.association import IAssociationContainer, IAssociationContainerTarget
from pyams_content.component.links.interfaces import ILinkContainerTarget
from pyams_content.reference.pictogram.interfaces import SELECTED_PICTOGRAM_VOCABULARY
from pyams_i18n.schema import I18nTextLineField
from pyams_sequence.interfaces import IInternalReference
from pyams_sequence.schema import InternalReferenceField


__docformat__ = 'restructuredtext'

from pyams_content import _


MENU_LINKS_CONTAINER_KEY = 'pyams_content.navigation'


class IMenuLink(Interface):
    """Menu link marker interface"""


class IMenuInternalLink(IMenuLink):
    """Menu internal link marker interface"""


class IMenuExternalLink(IMenuLink):
    """Menu external link marker interface"""


class IMenuMailtoLink(IMenuLink):
    """Menu mailto link marker interface"""


class IDynamicMenu(Interface):
    """Dynamic menu interface

    This interface is used to handle menus whose content is not static, but created
    dynamically based on its target contents.
    """


MENUS_CONTAINER_KEY = 'pyams_content.navigation'


class IMenuLinksContainer(IAssociationContainer):
    """Menu links container interface"""

    contains(IMenuLink)


class IMenuLinksContainerTarget(ILinkContainerTarget):
    """Menu links container target marker interface"""


MENU_ICON_CLASS = 'fas fa-table-list'
MENU_ICON_HINT = _("Navigation menu")


class IMenu(IMenuLinksContainerTarget, IInternalReference):
    """Menu container interface"""

    containers('.IMenusContainer')

    visible = Bool(title=_("Visible?"),
                   description=_("Is this item visible in front-office?"),
                   required=True,
                   default=True)

    def is_visible(self, request=None):
        """Is menu visible?"""

    title = I18nTextLineField(title=_("Menu title"),
                              description=_("Displayed menu label"),
                              required=True)

    reference = InternalReferenceField(title=_("Internal reference"),
                                       description=_("Direct reference to menu target"),
                                       required=False)

    dynamic_menu = Bool(title=_("Dynamic menu?"),
                        description=_("If 'yes', menu items will be built from internal "
                                      "reference navigation items; other static items will be "
                                      "placed after dynamic items"),
                        required=False,
                        default=False)

    force_canonical_url = Bool(title=_("Force canonical URL?"),
                               description=_("If 'yes', link to internal references will "
                                             "use their canonical instead of relative URL"),
                               required=True,
                               default=False)

    pictogram_name = Choice(title=_("Pictogram"),
                            description=_("Name of the pictogram associated with this menu; "
                                          "pictogram display may depend on selected renderer "
                                          "and skin"),
                            required=False,
                            vocabulary=SELECTED_PICTOGRAM_VOCABULARY)

    pictogram = Attribute("Pictogram object reference")

    def get_url(self, request=None, view_name=None):
        """Menu menu target URL"""


class IMenusContainer(IAssociationContainer):
    """Menus container interface"""

    contains(IMenu)


class IMenusContainerTarget(IAssociationContainerTarget):
    """Menus container target marker interface"""



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_content import _
from pyams_content.component.paragraph import IParagraphContainerTarget
from pyams_content.component.paragraph.portlet import IParagraphContainerPortletSettings, \
    IParagraphNavigationPortletSettings
from pyams_form.interfaces.form import IInnerSubForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortletPreviewer
from pyams_portal.zmi import PortletPreviewer
from pyams_portal.zmi.interfaces import IPortletConfigurationEditor
from pyams_portal.zmi.portlet import PortletConfigurationEditForm
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_zmi.interfaces import IAdminLayer


class ParagraphPortletSettingsEditFormMixin:
    """Paragraph portlet settings edit form mixin class"""

    @property
    def fields(self):
        fields = super().fields
        container = get_parent(self.context, IParagraphContainerTarget)
        if container is None:
            fields = fields.omit('paragraphs')
        return fields

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        paragraphs = self.widgets.get('paragraphs')
        if paragraphs is not None:
            paragraphs.no_value_message = _("No filter, all paragraphs are selected")
            paragraphs.prompt_message = _("No filter, all paragraphs are selected")
            paragraphs.placeholder = _("No filter, all paragraphs are selected")
        factories = self.widgets.get('factories')
        if factories is not None:
            factories.no_value_message = _("No filter, all paragraphs types are selected")
            factories.prompt_message = _("No filter, all paragraphs are selected")
            factories.placeholder = _("No filter, all paragraphs are selected")


#
# Paragraph container portlet settings configuration
#

@adapter_config(name='configuration',
                required=(IParagraphContainerPortletSettings, IAdminLayer,
                          IPortletConfigurationEditor),
                provides=IInnerSubForm)
class ParagraphContainerPortletSettingsEditForm(ParagraphPortletSettingsEditFormMixin,
                                                PortletConfigurationEditForm):
    """Paragraph container portlet settings edit form"""


@adapter_config(required=(Interface, IPyAMSLayer, Interface, IParagraphContainerPortletSettings),
                provides=IPortletPreviewer)
@template_config(template='templates/container-preview.pt', layer=IPyAMSLayer)
class ParagraphContainerPortletPreviewer(PortletPreviewer):
    """Paragraph container portlet previewer"""


#
# Paragraph navigation portlet settings configuration
#

@adapter_config(name='configuration',
                required=(IParagraphNavigationPortletSettings, IAdminLayer,
                          IPortletConfigurationEditor),
                provides=IInnerSubForm)
class ParagraphNavigationPortletSettingsEditForm(ParagraphPortletSettingsEditFormMixin,
                                                 PortletConfigurationEditForm):
    """Paragraph navigation portlet settings edit form"""


@adapter_config(required=(Interface, IPyAMSLayer, Interface, IParagraphNavigationPortletSettings),
                provides=IPortletPreviewer)
@template_config(template='templates/navigation-preview.pt', layer=IPyAMSLayer)
class ParagraphNavigationPortletPreviewer(PortletPreviewer):
    """Paragraph navigation portlet previewer"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.provider.youtube module

This is the YouTube external videos provider module.
"""

import re

from persistent import Persistent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.video import IExternalVideo, IExternalVideoSettings, external_video_settings
from pyams_content.component.video.interfaces import IExternalVideoProvider
from pyams_content.component.video.provider.interfaces import IYoutubeVideoSettings
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.registry import utility_config


__docformat__ = 'restructuredtext'

from pyams_content import _


YOUTUBE_BASE_URL = re.compile(r'https://youtu.be/(.*)')
YOUTUBE_EXTENDED_URL = re.compile(r'https://(?:www\.)?youtube.com/watch\?v=(.*)')


@factory_config(IYoutubeVideoSettings)
class YoutubeVideoSettings(Persistent):
    """Youtube video settings"""

    _video_id = FieldProperty(IYoutubeVideoSettings['video_id'])
    start_at = FieldProperty(IYoutubeVideoSettings['start_at'])
    stop_at = FieldProperty(IYoutubeVideoSettings['stop_at'])
    autoplay = FieldProperty(IYoutubeVideoSettings['autoplay'])
    loop = FieldProperty(IYoutubeVideoSettings['loop'])
    show_commands = FieldProperty(IYoutubeVideoSettings['show_commands'])
    hide_branding = FieldProperty(IYoutubeVideoSettings['hide_branding'])
    show_related = FieldProperty(IYoutubeVideoSettings['show_related'])
    allow_fullscreen = FieldProperty(IYoutubeVideoSettings['allow_fullscreen'])
    disable_keyboard = FieldProperty(IYoutubeVideoSettings['disable_keyboard'])
    width = FieldProperty(IYoutubeVideoSettings['width'])
    height = FieldProperty(IYoutubeVideoSettings['height'])

    @property
    def video_id(self):
        return self._video_id

    @video_id.setter
    def video_id(self, value):
        if value:
            match = YOUTUBE_BASE_URL.match(value) or YOUTUBE_EXTENDED_URL.match(value)
            if match:
                value = match.groups()[0]
        self._video_id = value


@utility_config(name='youtube',
                provides=IExternalVideoProvider)
class YoutubeVideoProvider:
    """YouTube video provider"""

    label = _("YouTube")
    weight = 10

    settings_interface = IYoutubeVideoSettings


@adapter_config(required=IExternalVideo,
                provides=IYoutubeVideoSettings)
def youtube_video_settings(context):
    """YouTube video settings factory"""
    return external_video_settings(context, provider_name='youtube')


@adapter_config(required=YoutubeVideoProvider,
                provides=IExternalVideoSettings)
def youtube_video_provider_settings_factory(context):
    """YouTube video provider settings factory"""
    return IYoutubeVideoSettings



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.navigation.portlet.zmi module

"""

from zope.interface import Interface, implementer

from pyams_content.component.association.interfaces import IAssociationInfo
from pyams_content.component.association.zmi.container import AssociationsGroup
from pyams_content.feature.navigation import IMenuLinksContainerTarget
from pyams_content.feature.navigation.portlet import IDoubleNavigationPortletSettings, \
    ISimpleNavigationPortletSettings
from pyams_form.interfaces.form import IInnerSubForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortletPreviewer
from pyams_portal.zmi import PortletPreviewer
from pyams_portal.zmi.interfaces import IPortletConfigurationEditor
from pyams_portal.zmi.portlet import PortletConfigurationEditForm
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IPropertiesEditForm

__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(Interface, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletPreviewer)
@template_config(template='templates/simple-preview.pt', layer=IPyAMSLayer)
class SimpleNavigationPortletPreviewer(PortletPreviewer):
    """Simple navigation portlet previewer"""

    @staticmethod
    def get_link_info(link):
        """Link information getter"""
        return IAssociationInfo(link)


@adapter_config(name='associations-group',
                required=(ISimpleNavigationPortletSettings, IAdminLayer, IPropertiesEditForm),
                provides=IInnerSubForm, force_implements=False)
@adapter_config(name='associations-group',
                required=(IMenuLinksContainerTarget, IAdminLayer, IPropertiesEditForm),
                provides=IInnerSubForm, force_implements=False)
class LinksContainerAssociationsGroup(AssociationsGroup):
    """Links container table group"""

    legend = _("Navigation links")


@adapter_config(required=(Interface, IPyAMSLayer, Interface, IDoubleNavigationPortletSettings),
                provides=IPortletPreviewer)
@template_config(template='templates/double-preview.pt', layer=IPyAMSLayer)
class DoubleNavigationPortletPreviewer(PortletPreviewer):
    """Double navigation portlet previewer"""

    @staticmethod
    def get_link_info(link):
        """Link information getter"""
        return IAssociationInfo(link)


@adapter_config(name='configuration',
                required=(IDoubleNavigationPortletSettings, IAdminLayer,
                          IPortletConfigurationEditor),
                provides=IInnerSubForm)
@implementer(IPropertiesEditForm)
class DoubleNavigationPortletSettingsEditForm(PortletConfigurationEditForm):
    """Double navigation portlet settings edit form"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.reference module

This module provides persistent classes and adapters used to handle views
internal references support and catalog-based queries.
"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import Eq, NotAny, NotEq
from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_catalog.query import CatalogResultSet
from pyams_content.shared.view import IViewSettings, IWfView
from pyams_content.shared.view.interfaces.query import EXCLUDED_VIEW_ITEMS, IViewQueryFilterExtension, \
    IViewQueryParamsExtension, IViewUserQuery
from pyams_content.shared.view.interfaces.settings import ALWAYS_REFERENCE_MODE, IViewInternalReferencesSettings, \
    ONLY_REFERENCE_MODE, VIEW_REFERENCES_SETTINGS_KEY
from pyams_sequence.interfaces import IInternalReferencesList, ISequentialIdInfo
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utility
from pyams_workflow.interfaces import IWorkflow, IWorkflowPublicationInfo, IWorkflowVersions

__docformat__ = 'restructuredtext'


@factory_config(IViewInternalReferencesSettings)
class ViewInternalReferencesSettings(Persistent, Contained):
    """View internal references settings"""

    select_context_references = FieldProperty(IViewInternalReferencesSettings['select_context_references'])
    references = FieldProperty(IViewInternalReferencesSettings['references'])
    references_mode = FieldProperty(IViewInternalReferencesSettings['references_mode'])
    exclude_context = FieldProperty(IViewInternalReferencesSettings['exclude_context'])

    @property
    def is_using_context(self):
        return self.select_context_references

    def get_references(self, context):
        refs = []
        if self.select_context_references:
            references = IInternalReferencesList(context, None)
            if ((references is not None) and
                    getattr(references, 'use_references_for_views', True) and
                    references.references):
                refs.extend(references.references)
        if self.references:
            refs.extend(self.references)
        return refs


@adapter_config(required=IWfView,
                provides=IViewInternalReferencesSettings)
@adapter_config(name='references',
                required=IWfView,
                provides=IViewSettings)
def view_internal_references_settings(view):
    """View internal references settings factory"""
    return get_annotation_adapter(view, VIEW_REFERENCES_SETTINGS_KEY,
                                  IViewInternalReferencesSettings,
                                  name='++view:references++')


@adapter_config(name='references',
                required=IWfView,
                provides=IViewQueryParamsExtension)
class ViewReferencesQueryParamsExtension(ContextAdapter):
    """View internal references query params extension"""

    weight = 10

    def get_params(self, context, request=None):
        """Query params getter"""
        settings = IViewInternalReferencesSettings(self.context)
        # check view references mode
        if settings.references_mode == ONLY_REFERENCE_MODE:
            # references are retrieved by query filter extension, so no params are required!
            yield None
        else:
            # check view settings
            if settings.exclude_context:
                sequence = ISequentialIdInfo(context, None)
                if sequence is not None:
                    oid = sequence.hex_oid
                    catalog = get_utility(ICatalog)
                    yield NotEq(catalog['oid'], oid)


@adapter_config(name='references',
                required=IWfView,
                provides=IViewQueryFilterExtension)
class ViewReferencesQueryFilterExtension(ContextAdapter):
    """View internal references filter extension

    If internal references are selected, these references are forced.
    """

    weight = 999

    def filter(self, context, items, request=None):
        """Filter incoming items based on references settings"""
        settings = IViewInternalReferencesSettings(self.context)
        references = settings.get_references(context)
        if not references:
            return items
        excluded_oid = None
        if settings.exclude_context:
            sequence = ISequentialIdInfo(context, None)
            if sequence is not None:
                excluded_oid = sequence.hex_oid
        if (not items) or (settings.references_mode in (ALWAYS_REFERENCE_MODE, ONLY_REFERENCE_MODE)):
            catalog = get_utility(ICatalog)
            for reference in reversed(references):
                if reference == excluded_oid:
                    continue
                params = Eq(catalog['oid'], reference)
                for item in CatalogResultSet(CatalogQuery(catalog).query(params)):
                    versions = IWorkflowVersions(item, None)
                    if versions is not None:
                        workflow = IWorkflow(item)
                        items.prepend(versions.get_versions(workflow.visible_states))
                    else:
                        publication_info = IWorkflowPublicationInfo(item, None)
                        if (publication_info is not None) and \
                                publication_info.is_visible(request):
                            items.prepend((item,))
        return items


@adapter_config(name='exclusions',
                context=IWfView,
                provides=IViewUserQuery)
class ExclusionsViewQueryParamsExtension(ContextAdapter):
    """Search folder exclusions for Elasticsearch

    This adapter is looking into request's annotations for items which should be excluded
    from search.
    """

    @staticmethod
    def get_user_params(request):
        excluded_items = request.annotations.get(EXCLUDED_VIEW_ITEMS)
        if excluded_items:
            catalog = get_utility(ICatalog)
            yield NotAny(catalog['oid'], excluded_items)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.alert.interfaces module

"""

from zope.interface import Interface

from pyams_content.shared.view import VIEW_CONTENT_TYPE
from pyams_sequence.schema import InternalReferenceField

__docformat__ = 'restructuredtext'

from pyams_content import _


ALERT_MANAGER_KEY = 'pyams_content.alerts'


class IAlertManagerInfo(Interface):
    """Alert manager info interface"""

    reference = InternalReferenceField(title=_("Global alerts view"),
                                       description=_("Internal view target reference; please note that alerts "
                                                     "content type selection will be added automatically to "
                                                     "settings of the selected view"),
                                       content_type=VIEW_CONTENT_TYPE,
                                       required=False)

    context_view = InternalReferenceField(title=_("Context alerts view"),
                                          description=_("Reference to the view used to get context alerts; please "
                                                        "note that alerts content type selection will be added "
                                                        "automatically to settings of the selected view"),
                                          content_type=VIEW_CONTENT_TYPE,
                                          required=False)

    def get_visible_alerts(self, request):
        """Iterator over visible alerts"""

    def get_context_alerts(self, request, context=None):
        """Iterator over visible alerts associated with request context"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.reference.pictogram.zmi.manager module

This module defines components which are used to handle pictograms selection in a
pictograms *manager*, which is used to select pictograms which will be available in a
given context.
"""

from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.reference.pictogram.interfaces import IPictogramManager, \
    IPictogramManagerTarget
from pyams_content.reference.pictogram.zmi.widget.manager import \
    PictogramManagerSelectionFieldWidget
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.interfaces.viewlet import IHelpViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='pictograms-manager.menu',
                context=IPictogramManagerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=760,
                permission=MANAGE_TOOL_PERMISSION)
class PictogramManagerMenu(NavigationMenuItem):
    """Pictogram manager menu"""

    label = _("Pictograms selection")
    href = '#pictograms-selection.html'


@ajax_form_config(name='pictograms-selection.html',
                  context=IPictogramManagerTarget, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
class PictogramManagerEditForm(AdminEditForm):
    """Pictogram manager selection form"""

    title = _("Pictograms management")
    legend = _("Selection of available pictograms")

    fields = Fields(IPictogramManager)
    fields['selected_pictograms'].widget_factory = PictogramManagerSelectionFieldWidget

    _edit_permission = MANAGE_TOOL_PERMISSION


@viewlet_config(name='languages.help',
                context=IPictogramManagerTarget, layer=IAdminLayer, view=PictogramManagerEditForm,
                manager=IHelpViewletManager, weight=10)
class PictogramManagerEditFormHelp(AlertMessage):
    """Pictogram manager edit form help"""

    _message = _("You can select which pictograms will be made available to contents published "
                 "inside this context.\n\n"
                 "To include a pictogram, you can move it from *available pictograms* list to "
                 "*selected pictograms* list by just doing a double click onto it; the reverse "
                 "operation is working in the same way...")

    message_renderer = 'markdown'



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.header.portlet module

"""

from zope.schema.fieldproperty import FieldProperty

from pyams_content.feature.header.portlet.interfaces import IPageHeaderPortletSettings, PAGE_HEADER_ICON_CLASS, \
    PAGE_HEADER_PORTLET_NAME
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(IPageHeaderPortletSettings)
class PageHeaderPortletSettings(PortletSettings):
    """Page header portlet settings"""

    display_logo = FieldProperty(IPageHeaderPortletSettings['display_logo'])
    display_context_title = FieldProperty(IPageHeaderPortletSettings['display_context_title'])
    display_profile_link = FieldProperty(IPageHeaderPortletSettings['display_profile_link'])


@portlet_config(permission=None)
class PageHeaderPortlet(Portlet):
    """Page header portlet"""

    name = PAGE_HEADER_PORTLET_NAME
    label = _("Page header")

    settings_factory = IPageHeaderPortletSettings
    toolbar_css_class = PAGE_HEADER_ICON_CLASS
